@@ -1,3 +1,4 @@
+/* -*- Mode: javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -21,6 +22,7 @@
  * Contributor(s):
  *   Doron Rosenberg <doronr@us.ibm.com>
  *   Alexander Sack <asac@jwsdot.com> - Canonical Ltd.
+ *   Chris Coulson <chris.coulson@canonical.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,7 +38,28 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-function nsPluginInstallerWizard(){
+const Cu = Components.utils;
+const Ci = Components.interfaces;
+const Cc = Components.classes;
+
+const nsIXULAppInfo             = Ci.nsIXULAppInfo;
+const nsIInterfaceRequestor     = Ci.nsIInterfaceRequestor;
+const nsIWebProgress            = Ci.nsIWebProgress;
+const nsIWebProgressListener    = Ci.nsIWebProgressListener;
+const nsISupportsWeakReference  = Ci.nsISupportsWeakReference;
+const nsISupports               = Ci.nsISupports;
+const nsIWebNavigation          = Ci.nsIWebNavigation;
+const nsIScriptSecurityManager  = Ci.nsIScriptSecurityManager;
+const nsIHttpProtocolHandler    = Ci.nsIHttpProtocolHandler;
+const nsIXULChromeRegistry      = Ci.nsIXULChromeRegistry;
+const nsIObserverService        = Ci.nsIObserverService;
+const nsISupportsPRBool         = Ci.nsISupportsPRBool;
+const nsIAppStartup             = Ci.nsIAppStartup;
+const nsIPluginHost             = Ci.nsIPluginHost;
+
+Cu.import("resource://ubufox/modules/PluginFinder.jsm");
+
+function nsPluginInstallerWizard() {
 
   // create the request array
   this.mPluginRequestArray = new Object();
@@ -44,8 +67,7 @@ function nsPluginInstallerWizard(){
   this.mPluginRequestArrayLength = 0;
 
   // create the plugin info array.
-  // a hash indexed by plugin id so we don't install 
-  // the same plugin more than once.
+  // a hash indexed by mimetype
   this.mPluginInfoArray = new Object();
   this.mPluginInfoArrayLength = 0;
 
@@ -58,9 +80,6 @@ function nsPluginInstallerWizard(){
   // array holding pids of plugins that require a license
   this.mPluginLicenseArray = new Array();
 
-  this.mPluginGroupBoxes = new Array();
-  this.mPluginPlaceHolder = null;
-
   // how many plugins are to be installed
   this.pluginsToInstallNum = 0;
 
@@ -87,61 +106,45 @@ function nsPluginInstallerWizard(){
 
   this.WSPluginCounter = 0;
   this.licenseAcceptCounter = 0;
-
-  this.prefBranch = null;
-
-  this.mNeedsRestart = false;
 }
 
-nsPluginInstallerWizard.prototype.getPluginData = function (){
-  // for each mPluginRequestArray item, call the datasource
+nsPluginInstallerWizard.prototype.getPluginData = function() {
   this.WSPluginCounter = 0;
 
-  // initiate the datasource call
-  var rdfUpdater = new nsRDFItemUpdater(this.getOS(), this.getChromeLocale());
-
-  for (var item in this.mPluginRequestArray) {
-    rdfUpdater.checkForPlugin(this.mPluginRequestArray[item]);
+  for (let mimetype in this.mPluginRequestArray) {
+    PluginFinder.getPluginInfo(mimetype, function(aMimeType, aPluginInfos) {
+      gPluginInstaller.pluginInfoReceived(aMimeType, aPluginInfos);
+    });
   }
 }
 
 // aPluginInfo is null if the datasource call failed, and pid is -1 if
 // no matching plugin was found.
-nsPluginInstallerWizard.prototype.pluginInfoReceived = function (aPluginRequestItem, aPluginInfos){
+nsPluginInstallerWizard.prototype.pluginInfoReceived = function(aMimeType,
+                                                                aPluginInfos) {
   this.WSPluginCounter++;
 
   if (aPluginInfos ) {
-    // hash by id
-
-    var resultSetMimeType = null;
-    var filteredPluginInfoSet = new Array();
     var noResultInfo = null;
-    var aPluginInfo = null;
-    for (var i = 0; i < aPluginInfos.length; i++) {
-      aPluginInfo = aPluginInfos[i];
-      resultSetMimeType = aPluginInfo.requestedMimetype;
-      if (aPluginInfo && aPluginInfo.pid != -1 &&
-	  ( aPluginInfo.XPILocation || aPluginInfo.manualInstallationURL || aPluginInfo.InstallerLocation)) {
-	hasResults = true;
-	filteredPluginInfoSet.push(aPluginInfo);
-	this.mPluginPidArray[aPluginInfo.pid] = aPluginInfo;
-      }
+    for each (let aPluginInfo in aPluginInfos) {
+    	this.mPluginPidArray[aPluginInfo.pid] = aPluginInfo;
     }
 
-    if(filteredPluginInfoSet.length > 0)
+    if(aPluginInfos.length > 0)
     {
-      this.mPluginInfoArray[aPluginRequestItem.mimetype] = filteredPluginInfoSet;
+      this.mPluginInfoArray[aMimeType] = aPluginInfos;
       this.mPluginInfoArrayLength++;
     } else {
-      this.mPluginNotFoundArray[aPluginRequestItem.mimetype] = aPluginRequestItem;
+      this.mPluginNotFoundArray[aMimeType] = true;
       this.mPluginNotFoundArrayLength++;
     }
   }
 
   var progressMeter = document.getElementById("ws_request_progress");
 
-  if (progressMeter.getAttribute("mode") == "undetermined")
+  if (progressMeter.getAttribute("mode") == "undetermined") {
     progressMeter.setAttribute("mode", "determined");
+  }
 
   progressMeter.setAttribute("value",
       ((this.WSPluginCounter / this.mPluginRequestArrayLength) * 100) + "%");
@@ -154,19 +157,17 @@ nsPluginInstallerWizard.prototype.pluginInfoReceived = function (aPluginRequestI
       // we want to allow user to cancel
       this.advancePage(null);
     }
-  } else {
-    // process more.
   }
 }
 
-nsPluginInstallerWizard.prototype.createPluginSetGroupBox = function (mimetype) {
+nsPluginInstallerWizard.prototype.createPluginSetGroupBox = function(mimetype) {
 
   var stringBundle = document.getElementById("ubufoxPluginWizardString");
   var gbox = document.createElement("vbox");
   gbox.setAttribute("flex", "1");
 
   var caption = document.createElement("caption");
-  caption.setAttribute("label", stringBundle.getString("ubufox.pluginWizard.availablePluginsPage.description.label")+" "+mimetype+":");
+  caption.setAttribute("label", stringBundle.getString("pluginwizard.available_plugins.description.label")+" "+mimetype+":");
   gbox.appendChild(caption);
 
   return gbox;
@@ -176,17 +177,7 @@ function doToggleInstallPluginEvent(e) {
   gPluginInstaller.toggleInstallPlugin(e);
 }
 
-nsPluginInstallerWizard.prototype.showPluginList = function () {
-  var toReplace = null;
-
-  if(this.mPluginGroupBoxes.length > 0)
-    this.mPluginGroupBoxes.pop();
-  
-  if (toReplace && this.mPluginPlaceHolder) {
-    toReplace.getParent().replaceChild(toReplace, 
-				       this.mPluginPlaceHolder);
-  }
-  
+nsPluginInstallerWizard.prototype.showPluginList = function() {
   this.pluginsToInstallNum = 0;
   var hasPluginWithInstallerUI = false;
 
@@ -200,13 +191,12 @@ nsPluginInstallerWizard.prototype.showPluginList = function () {
     groupBox = this.createPluginSetGroupBox(mimetype);
 
     if(firstPluginSet) {
-      this.mPluginPlaceHolder = document.getElementById("pluginselection-placeholder");
-      this.mPluginPlaceHolder.parentNode.replaceChild(groupBox, this.mPluginPlaceHolder);
+      placeHolder = document.getElementById("pluginselection-placeholder");
+      placeHolder.parentNode.replaceChild(groupBox, placeHolder);
     } else {
       lastSibling.parentNode.insertBefore(groupBox, lastSibling);
     }
     lastSibling = groupBox;
-    this.mPluginGroupBoxes.push(groupBox);
 
     var radiogroup = document.createElement("richlistbox");
     radiogroup.setAttribute("flex", "1");
@@ -215,8 +205,7 @@ nsPluginInstallerWizard.prototype.showPluginList = function () {
 
     // OK, lets add the "None" option first
     var first = true;
-    for (var i = 0; i < pluginInfoSet.length; i++) {
-      var pluginInfo = pluginInfoSet[i];
+    for each (let pluginInfo in pluginInfoSet) {
       var rli = document.createElement("richlistitem");
       rli.setAttribute("style", "border-bottom: dotted 1px lightgrey; padding: 0.5em 1em 0.5em 1em");
       rli._ubufoxPluginInfo = pluginInfo;
@@ -235,17 +224,14 @@ nsPluginInstallerWizard.prototype.showPluginList = function () {
       distributorImage.setAttribute("style", "vertical-align: middle");
       distributorImageWrap.setAttribute("flex", "0");
       distributorImage.setAttribute("class", "distributor-image");
-      if(pluginInfo.XPILocation && pluginInfo.XPILocation.indexOf("apt:") == 0) {
-	distributorImage.setAttribute("src", "chrome://ubufox/content/ubuntulogo32.png");
-	pluginInfo.IconUrl = "chrome://ubufox/content/ubuntulogo32.png";
-      } else {
-	if(!pluginInfo.IconUrl || pluginInfo.IconUrl.length == 0) {
-	  distributorImage.setAttribute("src", "chrome://ubufox/content/internet32.png");
-	  pluginInfo.IconUrl = "chrome://ubufox/content/internet32.png";
-	} else {
-	  distributorImage.setAttribute("src", pluginInfo.IconUrl);
-	}
-      } 
+      if (!pluginInfo.IconUrl || pluginInfo.IconUrl.length == 0) {
+        if (pluginInfo.type == "apt") {
+        	pluginInfo.IconUrl = "chrome://ubufox/content/ubuntulogo32.png";
+        } else {
+      	  pluginInfo.IconUrl = "chrome://ubufox/content/internet32.png";
+        }
+      }
+      distributorImage.setAttribute("src", pluginInfo.IconUrl);
 
       spacer = document.createElement("hbox");
       spacer.setAttribute("flex", "1");
@@ -263,33 +249,36 @@ nsPluginInstallerWizard.prototype.showPluginList = function () {
 
       var descDesc = document.createElement("description");
       if (!pluginInfo.desc) {
-        pluginInfo.desc = this.getString("ubufox.pluginWizard.description.notfound");
+        pluginInfo.desc = this.getString("pluginwizard.description.notfound");
       }
       descDesc.appendChild(document.createTextNode(pluginInfo.desc));
       descDesc.setAttribute("style", "white-space: pre;");
       nameAndDesc.appendChild(descDesc);
+
+      if (pluginInfo.InstallerShowsUI == "true") {
+        hasPluginWithInstallerUI = true;
+      }
     }
 
     radiogroup.selectedIndex = 0;
 
-    if (pluginInfo.InstallerShowsUI == "true")
-      hasPluginWithInstallerUI = true;
-
     // keep a running count of plugins the user wants to install
     this.pluginsToInstallNum++;
   }
 
-  if (hasPluginWithInstallerUI)
+  if (hasPluginWithInstallerUI) {
     document.getElementById("installerUI").hidden = false;
+  }
 
-  if (this.pluginsToInstallNum > 0)
+  if (this.pluginsToInstallNum > 0) {
     this.canAdvance(true);
-  else
+  } else {
     this.canAdvance(false);
+  }
   this.canRewind(false);
 }
 
-nsPluginInstallerWizard.prototype.toggleInstallPlugin = function (e) {
+nsPluginInstallerWizard.prototype.toggleInstallPlugin = function(e) {
   var selectedItem = e.target.selectedItem;
   var mime = selectedItem._ubufoxPluginInfoMimeType;
   this.mMimeTypePluginSelections[mime] = selectedItem._ubufoxPluginInfo.pid;
@@ -307,25 +296,25 @@ nsPluginInstallerWizard.prototype.toggleInstallPlugin = function (e) {
     this.canAdvance(false);
 }
 
-nsPluginInstallerWizard.prototype.canAdvance = function (aBool){
+nsPluginInstallerWizard.prototype.canAdvance = function(aBool) {
   document.getElementById("plugin-installer-wizard").canAdvance = aBool;
 }
 
-nsPluginInstallerWizard.prototype.canRewind = function (aBool){
+nsPluginInstallerWizard.prototype.canRewind = function(aBool) {
   document.getElementById("plugin-installer-wizard").canRewind = aBool;
 }
 
-nsPluginInstallerWizard.prototype.canCancel = function (aBool){
+nsPluginInstallerWizard.prototype.canCancel = function(aBool) {
   document.documentElement.getButton("cancel").disabled = !aBool;
 }
 
-nsPluginInstallerWizard.prototype.showLicenses = function (){
+nsPluginInstallerWizard.prototype.showLicenses = function() {
   this.canAdvance(false);
   this.canRewind(false);
 
   // only add if a license is provided and the plugin was selected to
   // be installed
-  for (var mimetype in this.mMimeTypePluginSelections){
+  for (var mimetype in this.mMimeTypePluginSelections) {
     var pid = this.mMimeTypePluginSelections[mimetype];
     var pluginInfo = this.mPluginPidArray[pid];
     if (pluginInfo && pluginInfo.licenseURL && (pluginInfo.licenseURL != "")) {
@@ -341,17 +330,17 @@ nsPluginInstallerWizard.prototype.showLicenses = function (){
 
     // add a nsIWebProgress listener to the license iframe.
     var docShell = document.getElementById("licenseIFrame").docShell;
-    var iiReq = docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
-    var webProgress = iiReq.getInterface(Components.interfaces.nsIWebProgress);
+    var iiReq = docShell.QueryInterface(nsIInterfaceRequestor);
+    var webProgress = iiReq.getInterface(nsIWebProgress);
     webProgress.addProgressListener(gPluginInstaller.progressListener,
-                                    Components.interfaces.nsIWebProgress.NOTIFY_ALL);
+                                    nsIWebProgress.NOTIFY_ALL);
 
 
     this.showLicense();
   }
 }
 
-nsPluginInstallerWizard.prototype.enableNext = function (){
+nsPluginInstallerWizard.prototype.enableNext = function() {
   // if only one plugin exists, don't enable the next button until
   // the license is accepted
   if (gPluginInstaller.pluginsToInstallNum > 1)
@@ -361,7 +350,6 @@ nsPluginInstallerWizard.prototype.enableNext = function (){
   document.getElementById("licenseRadioGroup2").disabled = false;
 }
 
-const nsIWebProgressListener = Components.interfaces.nsIWebProgressListener;
 nsPluginInstallerWizard.prototype.progressListener = {
   onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus)
   {
@@ -380,20 +368,20 @@ nsPluginInstallerWizard.prototype.progressListener = {
 
   QueryInterface : function(aIID)
   {
-     if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
-         aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
-         aIID.equals(Components.interfaces.nsISupports))
+     if (aIID.equals(nsIWebProgressListener) ||
+         aIID.equals(nsISupportsWeakReference) ||
+         aIID.equals(nsISupports))
        return this;
      throw Components.results.NS_NOINTERFACE;
    }
 }
 
-nsPluginInstallerWizard.prototype.showLicense = function (){
+nsPluginInstallerWizard.prototype.showLicense = function() {
   var pluginInfo = this.mPluginPidArray[this.mPluginLicenseArray[this.licenseAcceptCounter]];
 
   this.canAdvance(false);
 
-  var loadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
+  var loadFlags = nsIWebNavigation.LOAD_FLAGS_NONE;
 
   document.getElementById("licenseIFrame").webNavigation.loadURI(pluginInfo.licenseURL, loadFlags, null, null, null);
 
@@ -406,7 +394,7 @@ nsPluginInstallerWizard.prototype.showLicense = function (){
     pluginInfo.licenseAccepted ? 0 : 1;
 }
 
-nsPluginInstallerWizard.prototype.showNextLicense = function (){
+nsPluginInstallerWizard.prototype.showNextLicense = function() {
   var rv = true;
 
   if (this.mPluginLicenseArray.length > 0) {
@@ -425,7 +413,7 @@ nsPluginInstallerWizard.prototype.showNextLicense = function (){
   return rv;
 }
 
-nsPluginInstallerWizard.prototype.showPreviousLicense = function (){
+nsPluginInstallerWizard.prototype.showPreviousLicense = function() {
   this.storeLicenseRadioGroup();
   this.licenseAcceptCounter--;
 
@@ -440,7 +428,7 @@ nsPluginInstallerWizard.prototype.showPreviousLicense = function (){
   return false;
 }
 
-nsPluginInstallerWizard.prototype.storeLicenseRadioGroup = function (){
+nsPluginInstallerWizard.prototype.storeLicenseRadioGroup = function() {
   var pluginInfo = this.mPluginPidArray[this.mPluginLicenseArray[this.licenseAcceptCounter]];
   pluginInfo.licenseAccepted = !document.getElementById("licenseRadioGroup").selectedIndex;
 }
@@ -451,127 +439,56 @@ nsPluginInstallerWizard.prototype.licenseRadioGroupChange = function(aAccepted)
     this.canAdvance(aAccepted);
 }
 
-nsPluginInstallerWizard.prototype.advancePage = function (aPageId){
+nsPluginInstallerWizard.prototype.advancePage = function(aPageId) {
   this.canAdvance(true);
   document.getElementById("plugin-installer-wizard").advance(aPageId);
 }
 
-nsPluginInstallerWizard.prototype.startPluginInstallation = function (){
+nsPluginInstallerWizard.prototype.startPluginInstallation = function() {
   this.canAdvance(false);
   this.canRewind(false);
 
-  // for ubufox we deal with multiple cases: first case is XPIInstall, which will
-  // run the "normal" XPIInstall process; second case is XPIInstall url contains
-  // and apt: protocol url ... this will run apt protocol handler 
-  var installerPlugins = [];
-  var xpiPlugins = [];
-  this.aptPlugins = [];
+  let plugins = [];
 
-  for (var mime in this.mMimeTypePluginSelections) {
-    var pluginItem = this.mPluginPidArray[this.mMimeTypePluginSelections[mime]];
-
-    if (pluginItem && pluginItem.licenseAccepted) {
-      if (pluginItem.InstallerLocation)
-        installerPlugins.push(pluginItem);
-      else if (pluginItem.XPILocation && pluginItem.XPILocation.indexOf("apt:") != 0)
-        xpiPlugins.push(pluginItem);
-      else if (pluginItem.XPILocation && pluginItem.XPILocation.indexOf("apt:") == 0)
-        this.aptPlugins.push(pluginItem);
-      else {
-        window.alert("Unhandled mime install flavour (supported: vendor, apt)");
-        continue;
-      }
+  for each (let pid in this.mMimeTypePluginSelections) {
+    if (this.mPluginPidArray[pid].type != "manual") {
+      plugins.push(this.mPluginPidArray[pid]);
     }
   }
 
-  if (installerPlugins.length > 0 || xpiPlugins.length > 0)
-    PluginXPIInstallService.startPluginInstallation(installerPlugins,
-						    xpiPlugins);
-  else if (this.aptPlugins.length > 0)
-    PluginAPTInstallService.startPluginInstallation(this.aptPlugins);
-  else
+  if (plugins.length > 0) {
+    document.getElementById("plugin_install_progress")
+            .setAttribute("mode", "undetermined");
+    PluginFinder.install(plugins, this);
+  } else {
     this.advancePage(null);
-}
-
-/*
-  0    starting download
-  1    download finished
-  2    starting installation
-  3    finished installation
-  4    all done
-*/
-nsPluginInstallerWizard.prototype.pluginXPIInstallationProgress = function (aPid, aProgress, aError) {
-
-  var statMsg = null;
-  var pluginInfo = null;
-
-  if(aPid)
-    pluginInfo = gPluginInstaller.mPluginPidArray[aPid];
-
-  switch (aProgress) {
-
-    case 0:
-      statMsg = this.getFormattedString("pluginInstallation.download.start", [pluginInfo.name]);
-      break;
-
-    case 1:
-      statMsg = this.getFormattedString("pluginInstallation.download.finish", [pluginInfo.name]);
-      break;
-
-    case 2:
-      statMsg = this.getFormattedString("pluginInstallation.install.start", [pluginInfo.name]);
-      var progressElm = document.getElementById("plugin_install_progress");
-      progressElm.setAttribute("mode", "undetermined");
-      break;
-    case 6:
-      statMsg = "APT - " + this.getFormattedString("pluginInstallation.install.start", [pluginInfo.name]);
-      break;
-    case 3:
-      if (aError) {
-        statMsg = this.getFormattedString("pluginInstallation.install.error", [pluginInfo.name, aError]);
-        pluginInfo.error = aError;
-      } else {
-        statMsg = this.getFormattedString("pluginInstallation.install.finish", [pluginInfo.name]);
-        pluginInfo.error = null;
-      }
-      break;
-
-    case 7:
-      if (aError) {
-        statMsg = "APT - " + this.getFormattedString("pluginInstallation.install.error", [pluginInfo.name, aError]);
-        pluginInfo.error = aError;
-      } else {
-        statMsg = "APT - " + this.getFormattedString("pluginInstallation.install.finish", [pluginInfo.name]);
-        pluginInfo.error = null;
-      }
-      break;
-
-    case 4:
-      PluginAPTInstallService.startPluginInstallation(this.aptPlugins);
-      break;
-    case 8:
-      this.advancePage(null);
-      statMsg = this.getString("pluginInstallation.complete");
-      break;
-    default:
-      window.alert("unexpected error during plugin install [code=1001]");
-      break;
   }
-
-  if (statMsg)
-    document.getElementById("plugin_install_progress_message").value = statMsg;
 }
 
-nsPluginInstallerWizard.prototype.pluginInstallationProgressMeter = function (aPid, aValue, aMaxValue){
-  var progressElm = document.getElementById("plugin_install_progress");
+nsPluginInstallerWizard.prototype.onInstallStarted = function() {
 
-  if (progressElm.getAttribute("mode") == "undetermined")
-    progressElm.setAttribute("mode", "determined");
-  
-  progressElm.setAttribute("value", Math.ceil((aValue / aMaxValue) * 100) + "%")
 }
 
-nsPluginInstallerWizard.prototype.addPluginResultRow = function (aImgSrc, aName, aNameTooltip, aStatus, aStatusTooltip, aManualUrl){
+nsPluginInstallerWizard.prototype.onProgressChanged = function(aProgress) {
+  let elm = document.getElementById("plugin_install_progress");
+  elm.setAttribute("mode", "determined");
+  elm.setAttribute("value", Math.ceil(aProgress) + "%");
+}
+
+nsPluginInstallerWizard.prototype.onInstallFinished = function() {
+  this.advancePage(null);
+}
+
+nsPluginInstallerWizard.prototype.onPluginFailed = function(aPid, aError) {
+  this.mPluginPidArray[aPid].error = aError;
+  this.mPluginPidArray[aPid].failed = true;
+}
+
+nsPluginInstallerWizard.prototype.onPluginFinished = function(aPid) {
+  //this.mPluginPidArray[aPid].installed = true;
+}
+
+nsPluginInstallerWizard.prototype.addPluginResultRow = function(aImgSrc, aName, aNameTooltip, aStatus, aStatusTooltip, aManualUrl) {
   var myRows = document.getElementById("pluginResultList");
 
   var myRow = document.createElement("row");
@@ -587,13 +504,17 @@ nsPluginInstallerWizard.prototype.addPluginResultRow = function (aImgSrc, aName,
   // create the labels
   var myLabel = document.createElement("label");
   myLabel.setAttribute("value", aName);
-  if (aNameTooltip)
+  if (aNameTooltip) {
     myLabel.setAttribute("tooltiptext", aNameTooltip);
+  }
   myRow.appendChild(myLabel);
 
   if (aStatus) {
     myLabel = document.createElement("label");
     myLabel.setAttribute("value", aStatus);
+    if (aStatusTooltip) {
+      myLabel.setAttribute("tooltiptext", aStatusTooltip);
+    }
     myRow.appendChild(myLabel);
   }
 
@@ -617,44 +538,38 @@ nsPluginInstallerWizard.prototype.addPluginResultRow = function (aImgSrc, aName,
   myRows.appendChild(myRow);
 }
 
-nsPluginInstallerWizard.prototype.showPluginResults = function (){
+nsPluginInstallerWizard.prototype.showPluginResults = function() {
   var notInstalledList = "?action=missingplugins";
   var myRows = document.getElementById("pluginResultList");
 
-  // clear children
-  for (var run = myRows.childNodes.length; run--; run > 0)
-    myRows.removeChild(myRows.childNodes.item(run));
-
   for (var mimetype in this.mMimeTypePluginSelections) {
-    var pluginInfoItem = this.mPluginPidArray[this.mMimeTypePluginSelections[mimetype]];
+    var myPluginItem = this.mPluginPidArray[this.mMimeTypePluginSelections[mimetype]];
     // [plugin image] [Plugin_Name Plugin_Version] [Success/Failed] [Manual Install (if Failed)]
 
-    var myPluginItem = pluginInfoItem; //this.mPluginInfoArray[pluginInfoItem];
-
     var statusMsg;
     var statusTooltip;
-    if (myPluginItem.error){
+    if (myPluginItem.failed){
       statusMsg = this.getString("pluginInstallationSummary.failed");
       statusTooltip = myPluginItem.error;
-      notInstalledList += "&mimetype=" + pluginInfoItem;
+      notInstalledList += "&mimetype=" + mimetype;
     } else if (myPluginItem.licenseURL && !myPluginItem.licenseAccepted) {
       statusMsg = this.getString("pluginInstallationSummary.licenseNotAccepted");
-    } else if (!myPluginItem.XPILocation && !myPluginItem.InstallerLocation) {
-      statusMsg = this.getString("pluginInstallationSummary.notAvailable");
-      notInstalledList += "&mimetype=" + pluginInfoItem;
     } else {
       this.mSuccessfullPluginInstallation++;
       statusMsg = this.getString("pluginInstallationSummary.success");
 
       // only check needsRestart if the plugin was successfully installed.
-      if (myPluginItem.needsRestart)
-	this.mNeedsRestart = true;
+      if (myPluginItem.needsRestart) {
+      	this.mNeedsRestart = true;
+      }
     }
 
     // manual url - either returned from the webservice or the pluginspage attribute
     var manualUrl;
-    if ((myPluginItem.error || (!myPluginItem.XPILocation && !myPluginItem.InstallerLocation)) && (myPluginItem.manualInstallationURL || this.mPluginRequestArray[myPluginItem.requestedMimetype].pluginsPage)){
-      manualUrl = myPluginItem.manualInstallationURL ? myPluginItem.manualInstallationURL : this.mPluginRequestArray[myPluginItem.requestedMimetype].pluginsPage;
+    if (myPluginItem.error && this.mPluginRequestArray[mimetype].pluginsPage) {
+      manualUrl = this.mPluginRequestArray[mimetype].pluginsPage;
+    } else if (myPluginItem.type == "manual") {
+      manualUrl = myPluginItem.location;
     }
 
     this.addPluginResultRow(
@@ -667,21 +582,23 @@ nsPluginInstallerWizard.prototype.showPluginResults = function (){
   }
 
   // handle plugins we couldn't find
-  for (pluginInfoItem in this.mPluginNotFoundArray){
-    var pluginRequest = this.mPluginNotFoundArray[pluginInfoItem];
+  for (let mimetype in this.mPluginNotFoundArray) {
+    if (this.mPluginNotFoundArray[mimetype] == true) {
+      var pluginRequest = this.mPluginRequestArray[mimetype];
 
-    // if there is a pluginspage, show UI
-    if (pluginRequest.pluginsPage) {
-      this.addPluginResultRow(
-          "",
-          this.getFormattedString("pluginInstallation.unknownPlugin", [pluginInfoItem]),
-          null,
-          null,
-          null,
-          pluginRequest.pluginsPage);
+      // if there is a pluginspage, show UI
+      if (pluginRequest.pluginsPage) {
+        this.addPluginResultRow(
+            "",
+            this.getFormattedString("pluginInstallation.unknownPlugin", [mimetype]),
+            null,
+            null,
+            null,
+            pluginRequest.pluginsPage);
+      }
+
+      notInstalledList += "&mimetype=" + mimetype;
     }
-
-    notInstalledList += "&mimetype=" + pluginInfoItem;
   }
 
   // no plugins were found, so change the description of the final page.
@@ -696,8 +613,7 @@ nsPluginInstallerWizard.prototype.showPluginResults = function (){
 
   document.getElementById("pluginSummaryRestartNeeded").hidden = !this.mNeedsRestart;
 
-  var app = Components.classes["@mozilla.org/xre/app-info;1"]
-                      .getService(Components.interfaces.nsIXULAppInfo);
+  var app = Cc["@mozilla.org/xre/app-info;1"].getService(nsIXULAppInfo);
 
   // set the get more info link to contain the mimetypes we couldn't install.
   notInstalledList +=
@@ -726,7 +642,7 @@ nsPluginInstallerWizard.prototype.showPluginResults = function (){
   this.canRewind(false);
 }
 
-nsPluginInstallerWizard.prototype.loadURL = function (aUrl){
+nsPluginInstallerWizard.prototype.loadURL = function(aUrl) {
   // Check if the page where the plugin came from can load aUrl before
   // loading it, and do *not* allow loading URIs that would inherit our
   // principal.
@@ -734,10 +650,8 @@ nsPluginInstallerWizard.prototype.loadURL = function (aUrl){
   var pluginPagePrincipal =
     window.opener.content.document.nodePrincipal;
 
-  const nsIScriptSecurityManager =
-    Components.interfaces.nsIScriptSecurityManager;
-  var secMan = Components.classes["@mozilla.org/scriptsecuritymanager;1"]
-                         .getService(nsIScriptSecurityManager);
+  var secMan = Cc["@mozilla.org/scriptsecuritymanager;1"]
+               .getService(nsIScriptSecurityManager);
 
   secMan.checkLoadURIStrWithPrincipal(pluginPagePrincipal, aUrl,
     nsIScriptSecurityManager.DISALLOW_INHERIT_PRINCIPAL);
@@ -745,7 +659,7 @@ nsPluginInstallerWizard.prototype.loadURL = function (aUrl){
   window.opener.open(aUrl);
 }
 
-nsPluginInstallerWizard.prototype.getString = function (aName){
+nsPluginInstallerWizard.prototype.getString = function(aName) {
   var result;
   try {
     result = document.getElementById("pluginWizardString").getString(aName);
@@ -756,7 +670,7 @@ nsPluginInstallerWizard.prototype.getString = function (aName){
   return result;
 }
 
-nsPluginInstallerWizard.prototype.getFormattedString = function (aName, aArray){
+nsPluginInstallerWizard.prototype.getFormattedString = function (aName, aArray) {
   var result;
   try {
     result = document.getElementById("pluginWizardString").getFormattedString(aName, aArray);
@@ -767,73 +681,41 @@ nsPluginInstallerWizard.prototype.getFormattedString = function (aName, aArray){
   return result;
 }
 
-nsPluginInstallerWizard.prototype.getOS = function (){
-  var httpService = Components.classes["@mozilla.org/network/protocol;1?name=http"]
-                              .getService(Components.interfaces.nsIHttpProtocolHandler);
-  return httpService.oscpu;
+nsPluginInstallerWizard.prototype.getOS = function() {
+  return Cc["@mozilla.org/network/protocol;1?name=http"]
+            .getService(nsIHttpProtocolHandler).oscpu;
 }
 
-nsPluginInstallerWizard.prototype.getChromeLocale = function (){
-  var chromeReg = Components.classes["@mozilla.org/chrome/chrome-registry;1"]
-                            .getService(Components.interfaces.nsIXULChromeRegistry);
-  return chromeReg.getSelectedLocale("global");
+nsPluginInstallerWizard.prototype.getChromeLocale = function() {
+  return Cc["@mozilla.org/chrome/chrome-registry;1"]
+            .getService(nsIXULChromeRegistry).getSelectedLocale("global");
 }
 
-nsPluginInstallerWizard.prototype.getPrefBranch = function (){
-  if (!this.prefBranch)
-    this.prefBranch = Components.classes["@mozilla.org/preferences-service;1"]
-                                .getService(Components.interfaces.nsIPrefBranch);
-  return this.prefBranch;
-}
-function nsPluginRequest(aPlugRequest){
+function nsPluginRequest(aPlugRequest) {
   this.mimetype = encodeURI(aPlugRequest.mimetype);
   this.pluginsPage = aPlugRequest.pluginsPage;
 }
 
-function PluginInfo(aResult) {
-  this.name = aResult.name;
-  this.pid = aResult.pid;
-  this.version = aResult.version;
-  this.IconUrl = aResult.IconUrl;
-  this.InstallerLocation = aResult.InstallerLocation;
-  this.InstallerHash = aResult.InstallerHash;
-  this.XPILocation = aResult.XPILocation;
-  this.XPIHash = aResult.XPIHash;
-  this.InstallerShowsUI = aResult.InstallerShowsUI;
-  this.manualInstallationURL = aResult.manualInstallationURL;
-  this.requestedMimetype = aResult.requestedMimetype;
-  this.licenseURL = aResult.licenseURL;
-  this.needsRestart = (aResult.needsRestart == "true");
-
-  this.error = null;
-  this.toBeInstalled = true;
-
-  // no license provided, make it accepted
-  this.licenseAccepted = this.licenseURL ? false : true;
-}
-
 var gPluginInstaller;
 
-function wizardInit(){
+function wizardInit() {
   gPluginInstaller = new nsPluginInstallerWizard();
   gPluginInstaller.canAdvance(false);
   gPluginInstaller.getPluginData();
 }
 
-function wizardFinish(){
+function wizardFinish() {
   if (gPluginInstaller.mNeedsRestart) {
     // Notify all windows that an application quit has been requested.
-    var os = Components.classes["@mozilla.org/observer-service;1"]
-                       .getService(Components.interfaces.nsIObserverService);
-    var cancelQuit = Components.classes["@mozilla.org/supports-PRBool;1"]
-                               .createInstance(Components.interfaces.nsISupportsPRBool);
+    var os = Cc["@mozilla.org/observer-service;1"].getService(nsIObserverService);
+    var cancelQuit = Cc["@mozilla.org/supports-PRBool;1"]
+                        .createInstance(nsISupportsPRBool);
     os.notifyObservers(cancelQuit, "quit-application-requested", "restart");
 
     // Something aborted the quit process.
     if (!cancelQuit.data) {
-      var nsIAppStartup = Components.interfaces.nsIAppStartup;
-      var appStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
-                                 .getService(nsIAppStartup);
+      var appStartup = Cc["@mozilla.org/toolkit/app-startup;1"]
+                          .getService(nsIAppStartup);
       appStartup.quit(nsIAppStartup.eAttemptQuit | nsIAppStartup.eRestart);
       return true;
     }
@@ -845,9 +727,8 @@ function wizardFinish(){
 
     // reload plugins so JS detection works immediately
     try {
-      var ph = Components.classes["@mozilla.org/plugin/host;1"]
-                         .getService(Components.interfaces.nsIPluginHost);
-      ph.reloadPlugins(false);
+      Cc["@mozilla.org/plugin/host;1"].getService(nsIPluginHost)
+                                      .reloadPlugins(false);
     }
     catch (e) {
       // reloadPlugins throws an exception if there were no plugins to load
